<!-- $Id: database.inc,v 1.1 2001/09/04 06:30:59 james Exp james $ -->
<?php
#
#   database.inc, prepaid accounting, database access functions
#   Copyright (C) 2001  James Cameron (quozl@us.netrek.org)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Filesystem data storage
#
#	/var/issues
#	/var/issues/lock - lock file, always exists
#	/var/issues/new - temporary file
#	/var/issues/data - directory containing serialized arrays
#
#   Filesystem installation steps
#
#	mkdir /var/issues
#       mkdir /var/issues/data
#       touch /var/issues/new
#       touch /var/issues/lock
#       chown -R www-data:www-data /var/issues
#

include('common.inc');

# date functions

# return the default date
function date_default () {
    return time();
}

# translate a unix date to human readable ISO8601 format
function date_to_string ($date) {
    return date('Y-m-d H:i:s', $date);
}

# translate a human readable ISO8601 date to unix date
function date_from_string ($string) {
    list($date, $time) = explode(' ', $string);
    list($year, $month, $day) = explode('-', $date);
    list($hour, $minute, $second) = explode(':', $time);
    $date = mktime($hour, $minute, $second, $month, $day, $year);
    if (date_to_string($date) == $string) return $date;
    return 0;
}


# issue functions

# translate issue id to file path
function issue_to_file ($id) {
    return (PATH_DATA.$id);
}

# check if an issue exists, returns true or false
function issue_exists ($id) {
    return (is_file(issue_to_file($id)));
}

# return a list of issues known in the database
function issue_list () {
    $directory = PATH_DATA;
    $handle = opendir($directory);
    while (($file = readdir($handle)) !== false) {
	if (!is_file($directory.$file)) continue;
	$array[] = basename($file);
    }
    closedir($handle);

    if (gettype($array) == 'array') { sort($array); return $array; }

    return array();
}

# write an issue from array to file, with interlock
function issue_write ($issue) {

    # obtain interlock
    $lock = fopen(PATH_LOCK, 'a+');
    if (!flock($lock, 2)) {
	echo 'Failed to lock database';
	fclose($lock);
	return 0;
    }

    # open the file
    if (!($fp = fopen(PATH_NEW, 'w'))) {
	echo 'Failed to open database';
	fclose($lock);
	return 0;
    }

    # write the array to the file
    if (!fwrite($fp,serialize($issue))) {
	echo 'Failed to write to database';
	fclose($fp);
	fclose($lock);
	return 0;
    }

    # close the file 
    if (!fclose($fp)) {
	echo 'Failed to close database';
	fclose($lock);
	return 0;
    }

    # derive the file name
    $file = issue_to_file($issue['id']);

    # rename the new file into place
    if (!rename(PATH_NEW, $file)) {
	echo 'Failed to move database file into place';
	fclose($lock);
	return 0;
    }

    # release the lock and return success
    fclose($lock);
    return 1;
}

# read an issue from file into array
function issue_read ($id) {

    # derive the file name
    $file = issue_to_file($id);

    # return zero if no file
    if (!is_file($file)) { return 0; }

    # open the file
    if (!($fp = fopen($file, 'r'))) {
	echo 'Failed to open database';
	return 0;
    }

    # read the file
    $issue = unserialize(fread($fp, filesize($file)));

    # close the file
    fclose($fp);

    return $issue;
}

# return a default issue
function issue_default($id) {
    $issue['id'] = $id;
    $issue['status'] = '';
    $issue['priority'] = '';
    $issue['responsibility'] = '';
    $issue['next'] = 0;
    return $issue;
}

# allocate a new issue
function issue_allocate () {

    # obtain interlock
    $fp = fopen(PATH_NEXT, 'r+');
    if (!flock($fp, 2)) {
	echo 'Failed to lock next';
	fclose($fp);
	return 0;
    }

    $id = fgets($fp, 32);
    if (is_bool($id)) {
	if ($id == FALSE) {
	    echo 'Failed to read next';
	    fclose($fp);
	    return 0;
	}
    }

    $id = $id + 1;

    while(issue_exists($id)) {
	$id++;
    }
    
    if (fseek($fp, 0, SEEK_SET) != 0) {
	echo 'Failed to seek next';
	fclose($fp);
	return 0;
    }

    if (!fputs($fp, $id)) {
	echo 'Failed to write next';
	fclose($fp);
	return 0;
    }

    fclose($fp);
    return $id;
}

function head($name) {
    global $title;
    echo TITLE;
    echo '<br>';
    echo '
<table border=0><tr>
<td valign=top>
<img src=issues.jpg width=94 height=48 alt="Issues">
</td>
<td><img src=slash.jpg border=0></td>
<td valign=top>
<img border=0 src='.$name.' alt='.$name.'>
</td>
</tr></table>
';
}

function icon($link, $icon) {
    global $SCRIPT_NAME;

    if (eregi(basename($SCRIPT_NAME), $link)) {
	echo "<td valign=top align=center><a href=$link><img border=0 src=$icon alt=menu></a><br>^</td>";
    } else {
	echo "<td valign=top><a href=$link><img border=0 src=$icon alt=menu></a></td>";
    }
}

function slash() {
    echo '<td valign=top><img src=slash.jpg border=0></td>';
}

function menu() {
    echo '<center><table border=0><tr>';
    icon('list.phtml', 'list.jpg');
    slash();
    icon('add.phtml', 'add.jpg');
    echo '</tr></table></center>';
}

?>
